/* This is the windows-specific gp3 file. Compile and link with
gp3lib.c. Do not link with other files (such as gp3cygwin.c). 

*/

#include <windows.h>
#include <stdio.h>
#include "gp3lib.h"

#define DEBUGOUT 0
#if DEBUGOUT
#define DEBUG(tag,x) printf("DEBUG: %s=%02X\n",tag,x)
#else
#define DEBUG(tag,x)
#endif

/* This is the part that is platform specific */
/* This is for Windows */

static HANDLE comport;  // file descriptor


/* Open port at 57600 baud */
int gp3openport(char *port)
{
	DCB dcb;
	COMMTIMEOUTS cto;
	DWORD rv;
	comport=CreateFile(port,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	if (comport) 
	{
	   SetupComm(comport,128,128);
	   cto.ReadIntervalTimeout=0;
	   cto.ReadTotalTimeoutMultiplier=100;
	   cto.ReadTotalTimeoutConstant=0;
	   cto.WriteTotalTimeoutMultiplier=0;
	   cto.WriteTotalTimeoutConstant=100;
	   SetCommTimeouts(comport,&cto);
	   dcb.DCBlength=sizeof(dcb);
	   GetCommState(comport,&dcb);
	   rv=BuildCommDCB("baud=57600 parity=N data=8 stop=1",&dcb);
	   dcb.fBinary=TRUE;
	   dcb.fDtrControl=DTR_CONTROL_DISABLE;
	   dcb.fOutxDsrFlow=FALSE;
	   dcb.fNull=FALSE;
	   dcb.fRtsControl=RTS_CONTROL_ENABLE;
	   dcb.fOutxCtsFlow=TRUE;
	   dcb.fDsrSensitivity=FALSE;
	   dcb.fOutX=FALSE;
	   dcb.fInX=FALSE;
           rv=SetCommState(comport,&dcb);
	   if (!rv) 
	   {
		   CloseHandle(comport);
		   comport=NULL;
	   }
	}
	return comport!=NULL?0:-1;
}

/* Close the port when done */
void gp3closeport()
{
  CloseHandle(comport);
}

/* Write a byte to the GP3 */
int gp3write(int byte)
{
  char c=(char)byte;
  DWORD n=0;
  while (!gp3ready());
  DEBUG("write",byte);
  WriteFile(comport,&c,1,&n,NULL);
  return n==1;
}

/* Read a byte from the GP3 */
/* User may call this after repeat */
unsigned int gp3read(void)
{
  char c;
  DWORD n;
  unsigned int rv;
  ReadFile(comport,&c,1,&n,NULL);
  rv=((unsigned int)c) & 0xFF;;  
  DEBUG("read",rv);
  return (n==1)?rv:-1;
}

/* Check CTS (optional if you don't want to know if GP3 is busy.
   As written, the system will handle this automatically although
   your program will block until the GP3 is available. This will
   allow you to prevent that if you implement it.
   None of the portable code uses this!

   However, if you don't implement this, you'll need to remove
   the lines that call gp3ready from the example program.

   Not that your Linux/Unix or tty driver may not implement TIOCMGET
   or may not return correct results for this call. 
*/
int gp3ready(void)
{
  DWORD stat;
  Sleep(1); // wait 1ms in case GP3 is just starting a command
  GetCommModemStatus(comport,&stat);
  return (stat & MS_CTS_ON)!=0;
}





